@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2017,2018
+;;  Copyright (C) 2013,2014,2015,2017,2018,2021
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -33,6 +33,7 @@
             new-cookie
             request-cookies
             cookie-has-key?
+            get-value-from-cookie-name
             cookies-maker
             cookie-modify))
 
@@ -44,12 +45,12 @@
   (make-cookie nvp expires domain path secure http-only)
   cookie?
   (nvp cookie-nvp cookie-nvp!)          ; Name-Value-Pairs of the cookie
-  (expires cookie-expires cookie-expires!)    ; The expiration in Greenwich Mean Time
+  (expires cookie-expires cookie-expires!) ; The expiration in Greenwich Mean Time
   (domain cookie-domain cookie-domain!) ; The domain the cookie is good for
   (path cookie-path cookie-path!)       ; The path the cookie is good for
   ;; keep cookie communication limited to encrypted transmission
   (secure cookie-secure cookie-secure!) ; The secure need of cookie
-  (http-only cookie-httponly cookie-httponly!)); http-only
+  (http-only cookie-httponly cookie-httponly!)) ; http-only
 
 ;; NOTE: expires should be positive integer
 (define* (cookie-modify ck #:key expires domain path secure http-only)
@@ -89,9 +90,10 @@
                         (string-trim-both e)))
                   (string-split str #\;)))
          (nvp (let lp((rest ll) (result '()))
-                (cond ((or (null? rest) (is-cookie-keywords? (car rest)))
-                       result) ; drop the pair after keyword-value-pair
-                      (else (lp (cdr rest) (cons (car rest) result))))))
+                (cond
+                 ((or (null? rest) (is-cookie-keywords? (car rest)))
+                  result) ; drop the pair after keyword-value-pair
+                 (else (lp (cdr rest) (cons (car rest) result))))))
          (cookie (new-cookie #:expires (get-from-cookie ll "Expires")
                              #:path (get-from-cookie ll "Path")
                              #:domain (get-from-cookie ll "Domain")
@@ -119,18 +121,19 @@
 (define (generate-cookies cookies)
   (map (lambda (c) `(set-cookie . ,(cookie->header-string c))) cookies))
 
-(define *the-remove-expires* "Thu, 01-Jan-70 00:00:01 GMT")
+(define *the-remove-expires* "Thu, 01 Jan 1970 00:00:00 GMT")
 
 ;; NOTE: expires should be integer
 ;; NOTE: When we set #:expires to #t for removing the cookie, #:path MUST be "/".
 ;;       Otherwise the client may not clear the cookie correctly.
 (define* (new-cookie #:key (expires 3600) ; expires in seconds
                      (npv '())
-                     (path (and (eq? expires #t) "/")) (domain #f)
+                     (path #f) (domain #f)
                      (secure #f) (http-only #t))
-  (let ((e (cond ((integer? expires) (make-expires expires))
-                 ((eq? expires #t) *the-remove-expires*)
-                 (else #f)))) ; else #f for no expires
+  (let ((e (cond
+            ((eq? expires #t) *the-remove-expires*)
+            ((integer? expires) (make-expires expires))
+            (else #f)))) ; else #f for no expires
     (make-cookie npv e domain path secure http-only)))
 
 (define (cookie-set! cookie name value)
@@ -160,7 +163,8 @@
     ;;(format #t "cookies-str: ~a~%" cookies-str)
     (map header-string->cookie cookies-str)))
 
-(define (cookie-has-key? ck key)
-  (if (null? ck)
-      #f ; no cookie
-      (any (lambda (x) (and (cookie-ref x key) x)) ck)))
+(define (cookie-has-key? ckl key)
+  (any (lambda (x) (and (cookie-ref x key) x)) ckl))
+
+(define (get-value-from-cookie-name ckl key)
+  (any (lambda (x) (and=> (cookie-ref x key) car)) ckl))
