@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015,2016,2017,2018
+;;  Copyright (C) 2014,2015,2016,2017,2018,2019,2020
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -28,6 +28,7 @@
   #:re-export (artanis-version)
   #:export (*handlers-table*
             *conf-hash-table*
+            *http-options-table*
             *conn-pool*
             *before-response-hook*
             *after-request-hook*
@@ -35,15 +36,16 @@
             *after-websocket-handshake-hook*
             *DB-conn-init-hook*
             *when-sigint-hook*
+            *refresh-hook*
             *sql-mapping-lookup-table*
             *artanis-entry*
             %current-toplevel
             current-toplevel
             find-ENTRY-path
-            draw:is-dry-run?
-            draw:is-force?
-            draw:is-skip?
-            draw:is-quiet?
+            cmd:is-dry-run?
+            cmd:is-force?
+            cmd:is-skip?
+            cmd:is-quiet?
             artanis-current-output
             *controllers-table*
             current-dbconn
@@ -63,7 +65,16 @@
             current-encoder
             *lpc-instance-pool*
             out-of-task-prompt?
-            resources-collecting?))
+            resources-collecting?
+            oh-define!
+            oh-set!
+            oh-ref
+            is-mmapped-file?
+            register-mmapped-file!
+            http-status
+            get-syspage-handler
+            register-cache-handler!
+            has-cache-handler?))
 
 ;; WARNING:
 ;; For concurrency in green-thread, all these stuffs should be immutable
@@ -79,6 +90,10 @@
 (define *handlers-table* (make-hash-table))
 (define *conf-hash-table* (make-hash-table))
 
+;; table structure:
+;; ("<http-url-rules>" (<methods-list))
+(define *http-options-table* (make-hash-table))
+
 ;; NOTE: The init queue size equals to (server wqlen).
 ;;       If all the available DB conn were blocked, a new DB conn will be
 ;;       created, and never closed but just recycled by *conn-pool*.
@@ -90,6 +105,7 @@
 (define *DB-conn-init-hook* (make-hook 1))
 (define *after-websocket-handshake-hook* (make-hook 2))
 (define *when-sigint-hook* (make-hook))
+(define *refresh-hook* (make-hook))
 
 ;; TODO: I don't have much time for it but it should be RB-Tree in the future
 (define *sql-mapping-lookup-table* (make-hash-table))
@@ -119,16 +135,16 @@
       (find-ENTRY-path identity #t)))
 
 ;; parameters for command
-(define draw:is-dry-run? (make-parameter #f))
-(define draw:is-force? (make-parameter #f))
-(define draw:is-skip? (make-parameter #f))
-(define draw:is-quiet? (make-parameter #f))
+(define cmd:is-dry-run? (make-parameter #f))
+(define cmd:is-force? (make-parameter #f))
+(define cmd:is-skip? (make-parameter #f))
+(define cmd:is-quiet? (make-parameter #f))
 
 (define *null-device-output-port*
   (open-input-file *null-device*))
 
 (define (artanis-current-output)
-  (if (draw:is-quiet?)
+  (if (cmd:is-quiet?)
       *null-device-output-port*
       (current-output-port)))
 
@@ -212,3 +228,29 @@
 (define out-of-task-prompt? (make-parameter #f))
 
 (define resources-collecting? (make-parameter #f))
+
+(define *options-meta-handler-table* '())
+(define (oh-define! lst) (set! *options-meta-handler-table* lst))
+(define (oh-set! k h)
+  (set! *options-meta-handler-table*
+    (assq-set! *options-meta-handler-table* k h)))
+(define (oh-ref o)
+  (assq-ref *options-meta-handler-table* o))
+
+;; (client-fd . mmapped-bv)
+(define *mmapped-bv-from-file* (make-hash-table))
+(define (register-mmapped-file! fd bv)
+  (hashv-set! *mmapped-bv-from-file* fd bv))
+(define (is-mmapped-file? fd)
+  (hashv-ref *mmapped-bv-from-file* fd #f))
+
+(define *sys-page-handlers* (make-hash-table))
+(define (http-status status thunk)
+  (hashv-set! *sys-page-handlers* status thunk))
+(define (get-syspage-handler status)
+  (hashv-ref *sys-page-handlers* status #f))
+
+(define *cache-handlers-table* (make-hash-table))
+(define (register-cache-handler! rule handler)
+  (hash-set! *cache-handlers-table* rule handler))
+(define (has-cache-handler? uid) (hash-ref *cache-handlers-table* uid))
