@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015,2016,2017,2018,2019
+;;  Copyright (C) 2014,2015,2016,2017,2018,2019,2020,2021
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -27,7 +27,6 @@
   #:use-module (artanis env)
   #:use-module (artanis ssql)
   #:use-module (artanis fprm)
-  #:use-module (artanis crypto base64)
   #:use-module (artanis sql-mapping)
   #:use-module (artanis db)
   #:use-module (artanis cookie)
@@ -70,6 +69,8 @@
              :cookies-update!
              :cookies-remove!
              :cookies-setattr!
+             :cookies-check
+             :cookies-value
              :mime
              :auth
              :session
@@ -92,7 +93,8 @@
     (http-options-add! method rule)
     (hash-set! *handlers-table*
                (cons method path-regexp)
-               (make-handler-context handler keys (new-oht opts #:rule rule #:keys keys)))))
+               (make-handler-context handler keys (generate-rule-uid raw-rule)
+                                     (new-oht opts #:rule rule #:keys keys)))))
 
 (define (get rule . opts-and-handler) (define-handler 'GET rule opts-and-handler))
 (define (post rule . opts-and-handler) (define-handler 'POST rule opts-and-handler))
@@ -127,16 +129,11 @@
 (define-syntax-rule (=> opt rc args ...)
   (let* ((oht (rc-oht rc))
          (h (and oht (hash-ref oht opt))))
-    (if h
-        (h rc args ...)
-        (if (eq? opt #:cookies)
-            ;; #:cookies is special that no need to initialize first,
-            ;; and it could be used to operate the existing cookies
-            ;; from the requests.
-            values
-            (throw 'artanis-err 500 'oht-ref
-                   "The opt ~a isn't initialized for ~a"
-                   opt (rc-path rc))))))
+    (cond
+     (h (h rc args ...))
+     (else (throw 'artanis-err 500 'oht-ref
+                  "The opt ~a isn't initialized for ~a"
+                  opt (rc-path rc))))))
 
 (define-syntax-rule (auth-enabled? rc)
   (hash-ref (rc-oht rc) #:auth))
@@ -172,31 +169,32 @@
 
 ;; for #:cache
 (define (cache-maker pattern rule keys)
-  (define (non-cache rc body) body)
   (define-syntax-rule (try-public rc) (if (auth-enabled? rc) 'private 'public))
-  (match pattern
-    ;; disable cache explicitly, sometimes users want to make sure there's no any cache.
-    (#f non-cache)
-    (('static maxage0 ...)
-     (lambda* (rc #:key (dir #f) (maxage (->maxage maxage0)))
-       (let ((filename (if dir
-                           (format #f "~a/~a" dir (rc-path rc))
-                           (static-filename (rc-path rc)))))
-         (try-to-cache-static-file rc (static-filename (rc-path rc))
-                                   (try-public rc) maxage))))
-    (((? string? file) maxage0 ...)
-     (lambda* (rc #:key (maxage (->maxage maxage0)))
-       (try-to-cache-static-file rc file (try-public rc) maxage)))
-    (('public (? string? file) maxage0 ...)
-     (lambda* (rc #:key (maxage (->maxage maxage0)))
-       (try-to-cache-static-file rc file (try-public rc) maxage)))
-    (('private (? string? file) maxage0 ...)
-     (lambda* (rc #:key (maxage (->maxage maxage0)))
-       (try-to-cache-static-file rc file 'private maxage)))
-    ((or (? boolean? opts) (? list? opts))
-     (lambda (rc body) (try-to-cache-body rc body opts)))
-    (else (throw 'artanis-err 500 cache-maker
-                 "Invalid pattern!" pattern))))
+  (register-cache-handler!
+   (generate-rule-uid rule)
+   (match pattern
+     (('static maxage ...)
+      (lambda* (rc body #:key pre-headers status mtime request-status)
+        (let ((filename (static-filename (rc-path rc))))
+          (try-to-cache-static-file rc (static-filename (rc-path rc))
+                                    (try-public rc) (->maxage maxage)))))
+     (((? string? file) maxage ...)
+      (lambda* (rc body #:key pre-headers status mtime request-status)
+        (try-to-cache-static-file rc file (try-public rc) (->maxage maxage))))
+     (('public (? string? file) maxage ...)
+      (lambda* (rc body #:key pre-headers status mtime request-status)
+        (try-to-cache-static-file rc file (try-public rc) (->maxage maxage))))
+     (('private (? string? file) maxage ...)
+      (lambda* (rc body #:key pre-headers status mtime request-status)
+        (try-to-cache-static-file rc file 'private (->maxage maxage))))
+     ((or (? boolean? opts) (? list? opts))
+      (lambda* (rc body #:key (pre-headers '()) status mtime request-status)
+        (try-to-cache-body rc body #:cache-opts opts #:headers pre-headers)))
+     (else (throw 'artanis-err 500 cache-maker
+                  "Invalid pattern!" pattern))))
+  (lambda (rc . args)
+    (format (artanis-current-output)
+            "You don't have to call :cache explicitly!")))
 
 ;; for #:cookies
 ;; NOTE: Support cookies customization, which let users define special cookies.
@@ -214,7 +212,7 @@
   (define (cref ckl ck k)
     (and=> (cget ckl ck) (cut (current-cref) <> k)))
   (define (update ckl rc)
-    (rc-set-cookie! rc (map cdr ckl)))
+    (rc-set-cookie! rc (append (rc-set-cookie rc) (map cdr ckl))))
   (define (setattr ckl ck . kargs)
     (apply (current-setattr) (cget ckl ck) kargs))
   (define-syntax-rule (init-cookies names)
@@ -227,10 +225,13 @@
       ((set) (cut cset! ckl <> <> <>))
       ((ref) (cut cref ckl <> <>))
       ((setattr) (cut setattr ckl <> <...>))
+      ((check) (cut cookie-has-key? (rc-cookie rc) <>))
+      ((value) (cut get-value-from-cookie-name (rc-cookie rc) <>))
       ((update) (cut update ckl <>))
       (else (throw 'artanis-err 500 cookies-maker
                    "Invalid operation!" op))))
   (match mode
+    (#t (cut cookie-handler '() <> <>))
     (((or 'new 'names) names ...)
      (let ((ckl (init-cookies names)))
        (cut cookie-handler ckl <> <>)))
@@ -518,7 +519,7 @@
        (match (get-header rc 'authorization)
          ;; NOTE: In match `=' opetator, the order of evaluation is from left to right.
          ;;       So base64-decode will run first.
-         (`(basic . ,(= base64-decode-as-string (= (cut string-split <> #\:) up)))
+         (`(basic . ,(= nss:base64-decode (= (cut string-split <> #\:) up)))
           (let ((u (car up)) (p (cadr up)))
             (if checker
                 (checker rc u p)
@@ -776,7 +777,6 @@
 (meta-handler-register cookies)
 (meta-handler-register cache)
 (meta-handler-register mime)
-(meta-handler-register cookies)
 (meta-handler-register auth)
 (meta-handler-register session)
 (meta-handler-register from-post)
@@ -789,9 +789,17 @@
   ((:cookies rc 'ref) ck k))
 (define-syntax-rule (:cookies-setattr! rc ck kargs ...)
   ((:cookies rc 'setattr) ck kargs ...))
+(define-syntax-rule (:cookies-check rc name)
+  ((:cookies rc 'check) name))
+(define-syntax-rule (:cookies-value rc name)
+  ((:cookies rc 'value) name))
 (define-syntax-rule (:cookies-update! rc)
   ((:cookies rc 'update) rc))
-(run-before-response! (lambda (rc body) (:cookies-update! rc)))
+(run-before-response!
+ (lambda (rc body)
+   (when (and=> (rc-oht rc) (lambda (ht) (hash-ref ht #:cookies)))
+     ;; Call update iff the cookies is initialized
+     (:cookies-update! rc))))
 (define (:cookies-remove! rc k)
   (rc-set-cookie!
    rc
@@ -813,10 +821,18 @@
             (cond
              ((or h (eq? k #:handler))
               ;; #:handler is user customed handler
-              (when (eq? k #:with-auth)
-                ;; If #:session and #:with-auth are not initialized
-                ;; simultaneously, then init it with default option
+              (cond
+               ((or (eq? k #:auth) (eq? k #:with-auth))
+                ;; 1. If #:session and #:with-auth are not initialized
+                ;;    simultaneously, then init it with default option
+                ;; 2. Both require #:cookies
+                (when (not (hash-ref oht #:cookies))
+                  (init-oht! oht #:cookies #t rule keys))
                 (init-oht! oht #:session #t rule keys))
+               ((eq? k #:session)
+                ;; #:session requires #:cookies
+                (when (not (hash-ref oht #:cookies))
+                  (init-oht! oht #:cookies #t rule keys))))
               (hash-set! oht k (h v rule keys)))
              (else #f))))
         opts)
