@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2016,2017,2018
+;;  Copyright (C) 2016,2017,2018,2019
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -36,6 +36,7 @@
   #:use-module (artanis server server-context)
   #:use-module (artanis server scheduler)
   #:use-module (artanis server aio)
+  #:use-module (artanis websocket named-pipe)
   #:use-module ((srfi srfi-1) #:select (fold))
   #:use-module (system repl error-handling)
   #:use-module (srfi srfi-9)
@@ -75,14 +76,6 @@
   (DEBUG "generating work-table~%")
   (make-work-table (make-hash-table) (make-mutex)))
 
-(define *error-event* (logior EPOLLRDHUP EPOLLHUP))
-(define *read-event* EPOLLIN)
-(define (gen-read-event) (logior *read-event* (get-trigger)))
-(define *rw-event* (logior EPOLLIN EPOLLOUT))
-(define (gen-rw-event) (logior *error-event* (get-trigger) *rw-event*))
-(define *write-event* EPOLLOUT)
-(define (gen-write-event) (logior *error-event* (get-trigger) *write-event*))
-
 ;; Ragnarok server will establish a http-gateway, and support various protocols
 ;; based on websocket.
 ;; NOTE: Different from other ragnarok interfaces, ragnarok-open is not going to
@@ -130,38 +123,37 @@
     (task-kont task)
     ragnarok-scheduler))
 
-(define (is-task-timeout? task)
-  (let ((start-time (task-touch-time task))
-        (timeout (task-timeout task)))
-    (if (zero? timeout)
-        #f ; timeout = 0 means disable connection timeout
-        (>= (- (current-time) start-time) timeout))))
-
-(define (update-task-time! task)
-  (task-touch-time-set! task (current-time)))
-
 (define (resources-collector)
   (define (remove-timemout-connections)
-    (let ((wt (work-table-content (current-work-table (current-server))))
-          (http (current-proto))
-          (server (current-server)))
+    (let* ((server (current-server))
+           (wt (work-table-content (current-work-table server)))
+           (http (current-proto)))
       (hash-for-each
        (lambda (_ t)
          (catch 'resources-collector
            (lambda ()
              (when (is-task-timeout? t)
-               (throw 'resources-collector 408 serve-one-request
+               (format (artanis-current-output)
+                       "Collecting task ~a~%" t)
+               (throw 'resources-collector 408 remove-timemout-connections
                       "The request is timeout!")))
            (lambda e
              (call-with-values
                  (lambda ()
-                   (apply (make-unstop-exception-handler (exception-from-server)) e))
+                   (parameterize ((resources-collecting? #t))
+                     (apply (make-unstop-exception-handler exception-from-server) e)))
                (lambda (r b s)
-                 (ragnarok-write http server (task-client t) r b #f)
-                 (ragnarok-close http server (task-client t) #f))))))
+                 (catch #t
+                   (lambda ()
+                     (ragnarok-write http server (task-client t) r b #f)
+                     (ragnarok-close http server (task-client t) #f))
+                   (lambda _
+                     ;; ignore any error since there's no resource to handle.
+                     (remove-named-pipe-if-the-connection-is-websocket! (task-client t))
+                     (close-current-task! server (task-client t)))))))))
        wt)))
   ;; TODO: add more collectors
-  (when (> (get-conf '(server timeout)) 0)
+  (when (>= (get-conf '(server timeout)) 0)
     ;; If timeout is 0 then don't check timeout connections
     (remove-timemout-connections)))
 
@@ -177,7 +169,8 @@
 (define (no-available-port-to-allocate? e)
   (and (eq? (car e) 'system-error)
        (let ((errno (system-error-errno e)))
-         (= errno EADDRNOTAVAIL)))) ; no available port for connecting socket
+         (or (eqv? errno EADDRNOTAVAIL) ; no available port for connecting socket
+             (eqv? errno EMFILE))))) ; Too many opened files
 
 ;; NOTE: We will call `accept' if it's listenning socket. Logically, it's
 ;;       impossible to encounter listenning socket again while polling.
@@ -194,13 +187,16 @@
                      ((no-available-port-to-allocate? e)
                       (format (artanis-current-output)
                               "Ragnarok can't accept request. ~a~%"
-                              (strerror EADDRNOTAVAIL))
+                              (strerror (system-error-errno e)))
+                      (format (artanis-current-output)
+                              "Start resource collecting......")
                       (parameterize ((current-proto proto)
                                      (current-server server))
                         ;; When there's no available port for new requests,
                         ;; we call resources-collector to recycle timeout
                         ;; requests.
                         (resources-collector))
+                      (format (artanis-current-output) "done~%")
                       #f)
                      (else
                       (DEBUG "Get ~a new connections.~%" (length ret))
@@ -211,7 +207,7 @@
   (define (is-listenning-fd? e)
     (DEBUG "listenning-port? ~a~%" e)
     (let ((listen-socket (ragnarok-server-listen-socket server)))
-      (DEBUG "listen-socket: ~a = ~a~%" (car e) listen-socket)
+      (DEBUG "Is listen-socket? ~a ?= ~a~%" (car e) listen-socket)
       (= (car e) (port->fdes listen-socket))))
   ;;(DEBUG "Start to fill ready queue~%")
   ;;(print-work-table server)
@@ -225,22 +221,37 @@
        (let ((client
               (cond
                ((is-listenning-fd? e)
-                (DEBUG "New connection ~a~%" e)
+                (DEBUG "New connection from listening socket ~a~%" e)
                 (accept-them-all (ragnarok-server-listen-socket server)))
-               ((is-peer-shutdown? e)
-                (format #t "------peer shutdown~%")
-                (DEBUG "Peer shutdown ~a~%" e)
-                (parameterize ((must-close-connection? #t))
-                  (ragnarok-close
-                   proto
-                   server
-                   (restore-working-client (current-work-table server) (car e))
-                   #t)
-                  (DEBUG "Closed ~a~%" e)
-                  #f))
+               ((restore-working-client (current-work-table server) (car e))
+                => (lambda (client)
+                     (cond
+                      ((is-peer-shutdown? e)
+                       => (lambda (s)
+                            (DEBUG "Connecting socket ~a was shutdown!~%" e)
+                            (parameterize ((half-closed? s))
+                              (ragnarok-close
+                               proto
+                               server
+                               client
+                               #t)
+                              (if (half-closed?)
+                                  (DEBUG "Half-closed ~a~%" e)
+                                  (DEBUG "Full-closed ~a~%" e))
+                              #f)))
+                      (else
+                       (DEBUG "Restore working client ~a~%" e)
+                       client))))
+               ((exists-in-epoll? (ragnarok-server-epfd server) (car e))
+                (DEBUG "The fd ~a in still in epoll but not task for it, just ignore!~%"
+                       (car e))
+                (close (car e))
+                #f)
                (else
-                (DEBUG "Restore working client ~a~%" e)
-                (restore-working-client (current-work-table server) (car e))))))
+                (DEBUG "The fd ~a is neither in epoll, nor task for it, just ignore!~%"
+                       (car e))
+                (close (car e))
+                #f))))
          (cond
           ((list? client)
            (DEBUG "New coming connections: ~a~%" (length client))
@@ -249,7 +260,7 @@
            (DEBUG "Restored client ~a~%" (client-ip client))
            (ready-queue-in! rq client))
           (else
-           (DEBUG "The client ~a was shutdown~%" e)))))
+           (DEBUG "The client ~a is ready to shutdown~%" e)))))
      (epoll-wait epfd events timeout))))
 
 (define (handle-request handler request request-body)
@@ -278,13 +289,12 @@
 
 (define (is-listenning-socket? server client)
   (let ((listen-socket (ragnarok-server-listen-socket server)))
-    (= (port->fdes listen-socket) (client-sockport-decriptor client))))
+    (= (port->fdes listen-socket) (client-sockport-descriptor client))))
 
 (define (register-connecting-socket epfd conn-port)
   (DEBUG "Register ~a as RW event~%" conn-port)
   (epoll-ctl epfd EPOLL_CTL_ADD (port->fdes conn-port)
-             (make-epoll-event (port->fdes conn-port) (gen-rw-event))
-             #:keep-alive? #t)
+             (make-epoll-event (port->fdes conn-port) (gen-rw-event)))
   (DEBUG "register End~%"))
 
 (::define (serve-one-request handler proto server client)
@@ -320,7 +330,8 @@
                          (call-with-values
                              (lambda ()
                                (let ((task (current-task)))
-                                 (when (request-keep-alive? request)
+                                 (when (and (allow-long-live-connection?)
+                                            (request-keep-alive? request))
                                    (task-keepalive?-set! task #t)))
                                (handle-request handler request body))
                            (lambda (response body request-status)
@@ -330,15 +341,17 @@
                              (DEBUG "Ragnarok: write client~%")
                              (ragnarok-write proto server client response body
                                              (eq? 'HEAD (request-method request)))
-                             (cond
-                              ((or (eq? request-status 'exception)
-                                   (not (response-keep-alive? response))
-                                   (not (task-keepalive? (current-task))))
-                               (ragnarok-close proto server client #f))
-                              (else
-                               (DEBUG "Client ~a keep alive, status: ~a~%"
-                                      (client-sockport client) request-status)
-                               (kont)))))))))
+                             (let ((keepalive? (and (allow-long-live-connection?)
+                                                    (or (response-keep-alive? response)
+                                                        (task-keepalive? (current-task))))))
+                               (cond
+                                ((or (eq? request-status 'exception)
+                                     (not keepalive?))
+                                 (ragnarok-close proto server client #f))
+                                (else
+                                 (DEBUG "Client ~a keep alive, status: ~a~%"
+                                        (client-sockport client) request-status)
+                                 (kont))))))))))
              (conn (client-sockport client))
              (prio #t) ; TODO: we don't have prio yet
              (bufsize (get-conf '(server bufsize)))
@@ -384,13 +397,17 @@
   ((@ (web server) run-server)
    handler
    'http
-   (list #:port (get-conf '(host port)))))
+   (list #:port (get-conf '(host port))
+         #:family (get-family)
+         #:addr (get-addr))))
 
 (define (fibers-server-run handler)
   ((@ (web server) run-server)
    handler
    'fibers
-   (list #:port (get-conf '(host port)))))
+   (list #:port (get-conf '(host port))
+         #:family (get-family)
+         #:addr (get-addr))))
 
 (define (get-one-request-from-clients proto server)
   ;;(DEBUG "Prepare to get one request from clients, proto is ~a~%" proto)
@@ -401,6 +418,15 @@
       ;; with one epoll query.
       ;;(DEBUG "ready queue is empty~%")
       (fill-ready-queue-from-service proto server)
+      (when (null? rq)
+        (format (artanis-current-output)
+                "Start resource collecting......")
+        (parameterize ((current-proto proto)
+                       (current-server server))
+          ;; When there's no any request, the resources-collector will be triggered.
+          ;; This is necessary, since the timeouted connections can get 408 timely.
+          (resources-collector))
+        (format (artanis-current-output) "done~%"))
       ;;(DEBUG "fill ready queue from service~%")
       (get-one-request-from-clients proto server))
      (else
@@ -422,69 +448,104 @@
       (throw 'artanis-err 500 ragnarok-http-gateway-run
              "BUG: There should be `http' protocol at least!"))
     (let main-loop((client (get-one-request-from-clients http server)))
-      (let ((proto-name (detect-client-protocol client)))
-        (DEBUG "Enter main-loop, protocol is ~a~%" proto-name)
-        (call-with-sigint
-         ;; handle C-c to break the server loop properly
-         (lambda ()
-           (DEBUG "Prepare to serve one request ~a~%" (client-sockport client))
-           (catch 'uri-error
-             ;; NOTE: We make it very strict according to RFC 3986.
-             ;;       For example:
-             ;;       =======================3.3 Path============================
-             ;;       If a URI does not contain an authority component,
-             ;;       then the path cannot begin with two slash characters ("//").
-             ;;       ===========================================================
-             ;;       It means we will not accept the URL like:
-             ;;       http://localhost:3000//hello
-             ;;       And GNU Artanis will reply a warning page to tell
-             ;;       the client what's wrong with the URL.
-             ;;       Most mainstream server just try to make the client
-             ;;       happier even if the URL is wrong. I don't think
-             ;;       it's a good way to go. Make it strict to avoid
-             ;;       potential security risk is better.
-             (lambda ()
-               (catch 'artanis-err
-                 (lambda ()
-                   ;;(print-work-table server)
-                   (serve-one-request handler http server client))
-                 (lambda e
-                   (cond
-                    ((eqv? ETIMEDOUT (system-error-errno e))
-                     ;; NOTE: eqv? is necessary since system-error-errno on a
-                     ;;       non-system-erro will be non-integer, so don't use
-                     ;;       = to check.
-                     (main-loop (get-one-request-from-clients http server)))
-                    (else
-                     (call-with-values
-                         (lambda ()
-                           (apply (make-unstop-exception-handler (exception-from-server)) e))
-                       (lambda (r b s)
-                         (ragnarok-write http server client r b #f)
-                         (ragnarok-close http server client #f))))))))
-             (lambda (k . e)
-               (call-with-values
+      (catch #t
+        (lambda ()
+          (call-with-sigint
+           ;; handle C-c to break the server loop properly
+           (lambda ()
+             (DEBUG "Enter main-loop, protocol is ~a~%" (detect-client-protocol client))
+             (DEBUG "Prepare to serve one request ~a~%" (client-sockport client))
+             (catch 'uri-error
+               ;; NOTE: We make it very strict according to RFC 3986.
+               ;;       For example:
+               ;;       =======================3.3 Path============================
+               ;;       If a URI does not contain an authority component,
+               ;;       then the path cannot begin with two slash characters ("//").
+               ;;       ===========================================================
+               ;;       It means we will not accept the URL like:
+               ;;       http://localhost:3000//hello
+               ;;       And GNU Artanis will reply a warning page to tell
+               ;;       the client what's wrong with the URL.
+               ;;       Most mainstream server just try to make the client
+               ;;       happier even if the URL is wrong. I don't think
+               ;;       it's a good way to go. Make it strict to avoid
+               ;;       potential security risk is better.
+               (lambda ()
+                 (catch 'artanis-err
                    (lambda ()
-                     (let* ((reason (apply format #f e))
-                            (warn-page (get-syspage "warn-the-client.tpl"))
-                            (reason-page (string->bytevector
-                                          (tpl->html warn-page (the-environment))
-                                          (get-conf '(server charset))))
-                            (response (artanis-sys-response
-                                       400 (client-sockport client) reason-page)))
-                       (values response reason-page #f)))
-                 (lambda (r b s)
-                   (ragnarok-write http server client r b #f)
-                   (ragnarok-close http server client #f)))))
-           (DEBUG "Serve one done~%"))
-         (lambda ()
-           ;; NOTE: The remote connection will be handled gracefully in ragnarok-close
-           ;; NOTE: The parameters will be lost when exception raised here, so we can't
-           ;;       use any of current-task/server/client/proto in the exception handler
-           (DEBUG "Prepare to close connection ~a~%" (client-ip client))
-           (ragnarok-close http server client #f)))
-        (DEBUG "main-loop again~%")
-        (main-loop (get-one-request-from-clients http server))))))
+                     ;;(print-work-table server)
+                     (serve-one-request handler http server client))
+                   (lambda e
+                     (cond
+                      ((eqv? ETIMEDOUT (system-error-errno e))
+                       ;; NOTE: eqv? is necessary since system-error-errno on a
+                       ;;       non-system-erro will be non-integer, so don't use
+                       ;;       = to check.
+                       (main-loop (get-one-request-from-clients http server)))
+                      (else
+                       (call-with-values
+                           (lambda ()
+                             (apply (make-unstop-exception-handler exception-from-server) e))
+                         (lambda (r b s)
+                           (catch #t
+                             (lambda ()
+                               ;; NOTE: If we come here, then it's out of the task prompt,
+                               ;;       we must hold every kind of exception here to prevent
+                               ;;       the server down. Since it's out of the valid server
+                               ;;       continuation, we have no way to manage exceptions here
+                               ;;       but only ignore them.
+                               (parameterize ((out-of-task-prompt? #t))
+                                 (ragnarok-write http server client r b #f)
+                                 (ragnarok-close http server client #f)))
+                             (lambda e
+                               (DEBUG "An error occured outside of the task prmpt, ")
+                               (DEBUG "we have no choice but ignore it.~%")
+                               ;; NOTE: The error task must be removed here.
+                               (remove-named-pipe-if-the-connection-is-websocket! client)
+                               (close-current-task! server client))))))))))
+               (lambda (k . e)
+                 (call-with-values
+                     (lambda ()
+                       (let* ((reason (apply format #f e))
+                              (warn-page (get-syspage "warn-the-client.tpl"))
+                              (reason-page (string->bytevector
+                                            (tpl->html warn-page (the-environment))
+                                            (get-conf '(server charset))))
+                              (response (artanis-sys-response
+                                         400 (client-sockport client) reason-page)))
+                         (values response reason-page #f)))
+                   (lambda (r b s)
+                     (cond
+                      ((eqv? (cadr e) 408)
+                       (format (artanis-current-output)
+                               "Client `~a(~a)` was timeout, closed by server!"
+                               (client-sockport client) (client-ip client))
+                       (ragnarok-close http server client #f))
+                      (else
+                       (ragnarok-write http server client r b #f)
+                       (ragnarok-close http server client #f)))))))
+             (DEBUG "Serve one done~%"))
+           (lambda ()
+             ;; NOTE: The remote connection will be handled gracefully in ragnarok-close
+             ;; NOTE: The parameters will be lost when exception raised here, so we can't
+             ;;       use any of current-task/server/client/proto in the exception handler
+             (DEBUG "Prepare to close connection ~a~%" (client-ip client))
+             (ragnarok-close http server client #f))))
+        (lambda e
+          (format (artanis-current-output)
+                  "Error: ~a~%" (or (and=> (system-error-errno e) strerror) e))
+          (cond
+           ((out-of-system-resources? e)
+            (parameterize ((current-server server))
+              (remove-named-pipe-if-the-connection-is-websocket! client)
+              (close-current-task! server client)
+              (close (client-sockport client))
+              (resources-collector)))
+           (else
+            (format (artanis-current-output)
+                    "Ingore it to avoid Ragnarok crash.~%")))))
+      (DEBUG "main-loop again~%")
+      (main-loop (get-one-request-from-clients http server)))))
 
 ;; NOTE: we don't schedule guile-engine, although it provides naive mechanism for scheduling.
 (define (new-guile-engine)
@@ -582,37 +643,19 @@
      (else (error establish-http-gateway
                   "Invalid `server.engine' in artanis.conf" (ragnarok-engine-name engine))))))
 
-;; WARNING: Don't use = here, must use eqv? since it's not always numbers
-(define (io-exception:peer-is-shutdown? e)
-  (and (eq? (car e) 'system-error)
-       (let ((errno (system-error-errno e)))
-         (or (eqv? errno EPIPE) ; broken pipe
-             (eqv? errno EIO) ; write to a closed socket
-             (eqv? errno ECONNRESET))))) ; shutdown by peer
-
-;; WARNING: Don't use = here, must use eqv? since it's not always numbers
-(define (io-exception:out-of-memory? e)
-  (and (eq? (car e) 'system-error)
-       (let ((errno (system-error-errno e)))
-         (or (eqv? errno ENOMEM) ; no memory
-             (eqv? errno ENOBUFS))))) ; no buffer could be allocated
-
 (define (make-io-exception-handler type)
   (lambda e
     (DEBUG "ragnarok-~a exception: ~a~%" type e)
     (cond
      ((io-exception:peer-is-shutdown? e)
-      ;; NOTE: peer has been shutdown for reasons, we just let them be checked by epoll
-      ;;       in next round loop, and close the connection by Ragnarok.
-      ;; WARN: Don't call (close-task) directly, since we'll lose the chance to remove
-      ;;       it from epoll. Although epoll will remove closed fd automatically, it won't
-      ;;       remove the fd until it's closed completely, so does half-closed connection!
+      ;; WARN: Although epoll will remove closed fd automatically, it won't remove the fd
+      ;;       until it's closed completely, so does half-closed connection!
       ;;       Be careful.
-      (DEBUG "Peer is shutdown, just schedule and wait for closing it later ~%")
-      (break-task)
-      (DEBUG "Peer ~a is still open, we close it explicitly at the end!"
-             (client-sockport (current-client)))
-      (close-task))
+      (let ((conn-fd (client-sockport-descriptor (current-client)))
+            (epfd (ragnarok-server-epfd (current-server))))
+        (DEBUG "The closed peer ~a is going to be shut right now!~%" conn-fd)
+        (epoll-ctl epfd EPOLL_CTL_DEL conn-fd #f) ; #f means %null-pointer here
+        (close-task)))
      ((io-exception:out-of-memory? e)
       ;; NOTE: out of memory, and throw 503 to let client try again. We can't just schedule
       ;;       for next round loop, since some data read from request would be lost because
@@ -622,6 +665,13 @@
       (gc)
       (throw 'artanis-err 503 make-io-exception-handler
              "Ragnarok-~a: we run out of RAMs!~%" type))
+     ((out-of-system-resources? e)
+      => (lambda (reason)
+           (let ((shout (format #f
+                                "Out of system resources, maybe you should ~a!" reason)))
+             (display (WARN-TEXT shout) (artanis-current-output))
+             (resources-collector)
+             (break-task))))
      (else
       (DEBUG "Not an I/O exception, throw it to upper level~%")
       ;; not a exception should be handled here, re-throw it to the next level
